UNIT Shared;

INTERFACE

USES
  Windows, SysUtils;

CONST
  TXT_APP    = 'ImageJ';
  CFG_EXT    = '.cfg';

  JAR_IMAGEJ = 'ij.jar';
  JAR_TOOLS  = 'lib\tools.jar';

  EXE_JVM    = 'javaw.exe';

  DEF_JVM    = 'bin\'+EXE_JVM;
  DEF_JRE    = 'jre\'+DEF_JVM; // ImageJ installed
  DEF_CFG    = TXT_APP+CFG_EXT;
  DEF_PARAMS = '-Xmx%dm -cp %s'+JAR_IMAGEJ+' ij.ImageJ';

  NFG_JVM1   = '1.4.2'; // Exclude this if found

  REG_JRE    = '\SOFTWARE\JavaSoft\Java Runtime Environment';
  REG_JDK    = '\SOFTWARE\JavaSoft\Java Development Kit';
  REG_CUR    = 'CurrentVersion';
  REG_JHM    = 'JavaHome';

// Program functions
procedure FatalError(sMsg,sTitle: String; nErrCode: Integer);
function IJJarExists(sIJJar: String): Boolean;
function GetValidIJPath(var sIJPath: String; bDotFmt: Boolean): Boolean;
function GetValidJVM(sIJPath: String; var sJVMFile: String): Boolean;
function RelativeJVM(sIJPath,sJVM: String): String;
function CreateCfgFile(sDefCfg,sDefDir,sDefJVM,sDefParams: String;
  bPrompt: Boolean = false): String;
function ReadCfgFile(sCfgFile: String; var sGetDir,sGetJVM,sGetParams: String): Boolean;

// General utility functions
function EvalSet(bEval: Boolean; var bResult: Boolean): Boolean; overload;
function EvalSet(sEval: String; var sResult: String): String; overload;
function EvalSet(nEval: Integer; var nResult: Integer): Integer; overload;
function GetShortPathString(sLongPathName: String): String;
function GetSpecialFolderPath(nFolder: Integer; bTrailingSlash: Boolean = true): String;
function NormalizeDir(sPath: String; bTrailingSlash: Boolean = true): String;
function GetCmdLnParamsEx: String;

IMPLEMENTATION

uses
  ActiveX, ShlObj, Registry, OpenDlg;

// -----------------------------------------------------------------------

// Display error message and exit with error code
procedure FatalError(sMsg,sTitle: String; nErrCode: Integer);
begin
  MessageBox(0,PChar(sMsg),PChar(sTitle),MB_OK or MB_ICONERROR);
  halt(nErrCode);
end;

function IJJarExists(sIJJar: String): Boolean;
begin
  if not(EvalSet(FileExists(sIJJar),Result)) then
    Result:=FileExists(NormalizeDir(sIJJar)+JAR_IMAGEJ);
end;

function GetValidIJPath(var sIJPath: String; bDotFmt: Boolean): Boolean;
begin
  if (EvalSet(Trim(sIJPath),sIJPath) = '') or (sIJPath = '.') then
    sIJPath:=ExtractFilePath(ParamStr(0));
  sIJPath:=NormalizeDir(sIJPath)+JAR_IMAGEJ;
  // If no file, then try default install folder...
  if not(FileExists(sIJPath)) then
    begin
      sIJPath:=GetSpecialFolderPath(CSIDL_PROGRAM_FILES)+TXT_APP+'\'+JAR_IMAGEJ;
      // If no file, then setup default search location
      if not(FileExists(sIJPath)) then
        sIJPath:=GetSpecialFolderPath(CSIDL_PROGRAM_FILES)+JAR_IMAGEJ;
    end;
  if FileExists(sIJPath) or
     OpenDialog(0,'Select the '+TXT_APP+' JAR File',ExtractFilePath(sIJPath),
                TXT_APP+' ('+JAR_IMAGEJ+')|'+JAR_IMAGEJ+'|All Files (*.*)|*.*',sIJPath) then
    begin
      sIJPath:=ExtractFilePath(sIJPath);
      if bDotFmt and SameText(sIJPath,ExtractFilePath(ParamStr(0))) then
        sIJPath:='.';
      Result:=true;
    end
  else
    Result:=false;
end;

function GetValidJVM(sIJPath: String; var sJVMFile: String): Boolean;
var
  reg: TRegistry;
  sJVM: String;

function GetJPath(sRegPath: String; var sPath: String): Boolean;
var
  sVersion: String;
begin
  Result:=false;
  if reg.OpenKey(REG_JRE,false) then
    try
      sPath:=reg.ReadString(REG_CUR);
      reg.CloseKey;
      if reg.OpenKey(NormalizeDir(sRegPath)+sPath,false) then
        begin
          sPath:=NormalizeDir(reg.ReadString(REG_JHM))+DEF_JVM;
          // Exclude NFG_JVM1 here (list could be longer)
          Result:=(Pos(NFG_JVM1,sPath) = 0) and FileExists(sPath);
          reg.CloseKey;
        end;
    except
      reg.CloseKey;
    end;
end;

begin
  // Check if file exists outright
  if EvalSet(FileExists(sJVMFile),Result) then // Note: sets Result value here
    exit;
  // Check if it is in a relative path
  if (sIJPath = '') or (sIJPath = '.') then
    sIJPath:=ExtractFilePath(ParamStr(0));
  sIJPath:=NormalizeDir(sIJPath);
  if (Pos(':',sJVMFile) = 0) and FileExists(sIJPath+sJVMFile) then
    begin
      sJVMFile:=sIJPath+sJVMFile; // Expand the path
      Result:=true;
      exit;
    end;
  // It doesn't exist and is not in a relative path - default to ImageJ JRE
  if FileExists(sIJPath+DEF_JRE) then
    begin
      sJVMFile:=sIJPath+DEF_JRE;
      Result:=true;
      exit;
    end;
  // Still nothing - check the registry to auto-detect
  try
    reg:=TRegistry.Create;
    reg.RootKey:=HKEY_LOCAL_MACHINE;
    // Get the JRE location - removed "choice" logic here
    // Now it ALWAYS defaults to JDK (Wayne's request :-)
    //
    // Try JDK first
    if GetJPath(REG_JDK,sJVM) then
      begin
        sJVMFile:=sJVM;
        Result:=true;
        exit;
      end;
    // Try JRE next
    if GetJPath(REG_JRE,sJVM) then
      begin
        sJVMFile:=sJVM;
        Result:=true;
        exit;
      end;
  finally
    reg.Free;
  end;
  // Could not find JVM in registry - ask user
  sJVMFile:=GetSpecialFolderPath(CSIDL_PROGRAM_FILES)+EXE_JVM;
  Result:=OpenDialog(0,'Select the JVM for '+TXT_APP,ExtractFilePath(sJVMFile),
                     'JVM ('+EXE_JVM+')|'+EXE_JVM+'|All Files (*.*)|*.*',sJVMFile);
end;

function RelativeJVM(sIJPath,sJVM: String): String;
begin
  if (EvalSet(Trim(sIJPath),sIJPath) = '') or (sIJPath = '.') then
    sIJPath:=ExtractFilePath(ParamStr(0))
  else
    sIJPath:=NormalizeDir(sIJPath);
  sJVM:=Trim(sJVM);
  if (Pos(UpperCase(sIJPath),Uppercase(sJVM)) = 1) then
    Result:=Copy(sJVM,Length(sIJPath)+1,Length(sJVM))
  else
    Result:=sJVM;
end;

function CreateCfgFile(sDefCfg,sDefDir,sDefJVM,sDefParams: String;
  bPrompt: Boolean = false): String;
var
  fCfg: Text;
  bCleanup: Boolean;
  mst: TMemoryStatus;
  sJTools: String;
begin
  bCleanup:=false;
  if (Trim(sDefCfg) = '') then
    Result:=ChangeFileExt(ParamStr(0),CFG_EXT)
  else
    Result:=sDefCfg;
  if bPrompt then
    MessageBox(0,'The program will now be auto-configured. You may be prompted '+
               'to input the locations of various files that can not be auto-detected. '+
               'Please consult the installation instructions for further assistance.',
               PChar('Welcome to '+TXT_APP),MB_OK or MB_ICONINFORMATION);
  try
    assignFile(fCfg,Result);
    rewrite(fCfg);
    bCleanup:=true;
    // Line 1
    if GetValidIJPath(sDefDir,true) then // Returns '.' if possible
      writeln(fCfg,sDefDir)
    else
      Abort;
    // Line 2
    if GetValidJVM(sDefDir,sDefJVM) then
      writeln(fCfg,RelativeJVM(sDefDir,sDefJVM)) // Save as relative path
    else
      Abort;
    // Line 3
    sDefParams:=Trim(sDefParams);
    // Always check for tools.jar - if we have it, use it
    sJTools:=ExtractFilePath(sDefJVM); // Get folder
    Delete(sJTools,Length(sJTools),1); // Remove trailing '\'
    sJTools:=ExtractFilePath(sJTools)+JAR_TOOLS;
    if FileExists(sJTools) then
      sJTools:=GetShortPathString(sJTools)+';'
    else
      sJTools:='';
    // Check if Params is empty or if tools.jar is not there and set it
    if (sDefParams = '') or
       ((sJTools <> '') and (Pos(UpperCase(sJTools),UpperCase(sDefParams)) = 0)) then
      begin
        mst.dwLength:=SizeOf(mst);
        GlobalMemoryStatus(mst);
        // Default to 2/3 available physical memory in MB
        sDefParams:=Format(DEF_PARAMS,[((mst.dwTotalPhys div $300000)*2),sJTools]);
      end;
    writeln(fCfg,sDefParams);
    closeFile(fCfg);
    // Notify the user when a configuration file has been create
    MessageBox(0,PChar('A configuration file was successfully created as:'+#13#13+
               Result+#13#13+'with the following parameters:'+#13#13+
              'Line 1-  '+sDefDir+#13+
              'Line 2-  '+RelativeJVM(sDefDir,sDefJVM)+#13+
              'Line 3-  '+sDefParams+#13#13+
              'Please consult the installation instructions for further details.'),
              PChar(TXT_APP+' Configuration'),MB_OK or MB_ICONINFORMATION);
  except
    if bCleanup then
      begin
        closeFile(fCfg);
        DeleteFile(Result); // Cleanup
      end;
    FatalError('Unable to create the configuration file:'+#13#13+Result+#13#13+
               'Please check the installation folder and try again.',
               'Configuration File Error',1);
  end;
end;

function ReadCfgFile(sCfgFile: String; var sGetDir,sGetJVM,sGetParams: String): Boolean;
var
  fCfg: Text;
  bCleanup: Boolean;
begin
  Result:=false;
  bCleanup:=false;
  // Read the config file for info - load each string then trim whitespace chars
  try
    assignFile(fCfg,sCfgFile);
    reset(fCfg);
    bCleanup:=true;
    readln(fCfg,sGetDir);
    sGetDir:=Trim(sGetDir);
    if (sGetDir = '') or (sGetDir = '.') then
      sGetDir:=ExtractFilePath(ParamStr(0));
    readln(fCfg,sGetJVM);
    sGetJVM:=Trim(sGetJVM);
    readln(fCfg,sGetParams);
    sGetParams:=Trim(sGetParams);
    if (sGetParams = '') then // Missing JVM or Parameters?
      Abort;
    closeFile(fCfg);
    Result:=true; // All OK
  except
    if bCleanup then
      closeFile(fCfg);
  end;
end;

// -----------------------------------------------------------------------

function EvalSet(bEval: Boolean; var bResult: Boolean): Boolean;
begin
  bResult:=bEval;
  Result:=bEval;
end;

function EvalSet(sEval: String; var sResult: String): String;
begin
  sResult:=sEval;
  Result:=sEval;
end;

function EvalSet(nEval: Integer; var nResult: Integer): Integer;
begin
  nResult:=nEval;
  Result:=nEval;
end;

function GetShortPathString(sLongPathName: String): String;
var
  nLen: Integer;
begin
  Result:=sLongPathName;
  if (EvalSet(GetShortPathName(PChar(sLongPathName),nil,0),nLen) > 0) then
    begin
      SetLength(Result,nLen-1);
      GetShortPathName(PChar(sLongPathName),PChar(Result),nLen);
    end;
end;

function FreePIDL(var pidl: PItemIdList): Boolean;
var
  pMalloc: IMalloc;
begin
  pMalloc:=nil;
  if Assigned(pidl) then
    begin
      // Not sure if DidAlloc code is really necessary?  2004-02-01
      if Succeeded(SHGetMalloc(pMalloc)) then  // and (pMalloc.DidAlloc(pidl) > 0) then
        begin
          pMalloc.Free(pidl);
          pidl:=nil;
          Result:=true;
        end
      else
        Result:=false;
    end
  else
    Result:=true;
end;

function GetSpecialFolderPath(nFolder: Integer; bTrailingSlash: Boolean = true): String;
var
  pidl: PItemIDList;
  szPath: Array[0..MAX_PATH] of Char;
begin
  Result:='';
  pidl:=nil;
  FillChar(szPath,SizeOf(szPath),#0);
  if Succeeded(SHGetSpecialFolderLocation(0,nFolder,pidl)) then
    begin
      SHGetPathFromIDList(pidl,szPath);
      FreePIDL(pidl);
    end;
  Result:=Trim(szPath);
  if bTrailingSlash and (Result <> '') and (Result[Length(Result)] <> '\') then
    Result:=Result+'\';
end;

function NormalizeDir(sPath: String; bTrailingSlash: Boolean = true): String;
var
  sRoot: String;
  nPos: Integer;
begin
  Result:=Trim(sPath);
  if (Result <> '') then
    begin
      while (Pos('/',Result) <> 0) do
        Result[Pos('/',Result)]:='\';
      // This prevents path from ending in multi-"\"
      while (Result <> '') and (Result[Length(Result)] = '\') do
        Delete(Result,Length(Result),1);
      if bTrailingSlash then
        Result:=Result+'\';
      while (EvalSet(Pos('\..',Result),nPos) <> 0) and
            (Pos('\',EvalSet(Copy(Result,1,nPos-1),sRoot)) <> 0) do
        begin
          while (sRoot <> '') and (sRoot[Length(sRoot)] <> '\') do
            Delete(sRoot,Length(sRoot),1);
          Result:=sRoot+Copy(Result,nPos+4,Length(Result)-nPos); // 4 = Length('\..')+1
        end;
    end;
end;

// Parse command line for passed files
function GetCmdLnParamsEx: String;
var
  i: Integer;
begin
  Result:='';
  if (ParamCount > 0) then
    begin
      // Expand the filenames on the command-line - default to quote everything
      for i:=1 to ParamCount do
        Result:=Result+' "'+ExpandFileName(ParamStr(i))+'"';
      Result:=Trim(Result);
    end;
end;

INITIALIZATION
  CoInitialize(nil);
FINALIZATION
  CoUninitialize;
END.
